import pickle
import pkgutil


class Exim:
    file_1 = pkgutil.get_data(__name__,"files/all_birds_list")
    #file_1 = open("all_birds_list", 'rb')
    all_birds_list = pickle.loads(file_1)
    all_birds_list_padded = []
    for bird in all_birds_list:
        all_birds_list_padded.append(" " + bird + " ")

    file_2 = pkgutil.get_data(__name__,"files/birdnames_words")
    #file_2 = open("birdnames_words", 'rb')
    birdnames_words = pickle.loads(file_2)

    def __init__(self):
        pass



import spacy
from IndiAvesBirdIdentification.exim import Exim


class BirdDetection:
    bird_list = []

    def find_bird_from_list(self,sentence):
        exim_ = Exim()
        all_birds_list = exim_.all_birds_list_padded
        bird_list = []
        sentence = " " + sentence + " "  # padding
        for bird in all_birds_list:
            if sentence.find(bird) > -1:
                bird_list.append(bird.strip())
        return bird_list


    def run_model(self, sentence):
        result_ = []
        nlp_ner = spacy.load("./model-best")
        doc = nlp_ner(sentence)
        for ent in doc.ents:
            result_.append(ent)
        return result_


from IndiAvesBirdIdentification.preprocess import UserSentence
from IndiAvesBirdIdentification.bird_detection import BirdDetection


class Get_Bird_Names:
    def merge_lists(self, list1, final_list):
        for i in list1:
            if type(i) != str:
                i = str(i)
            if i not in final_list:
                final_list.append(i)
        return final_list

    def consolidate_bird_list(self, list1, list2):
        total_list = []
        total_list = self.merge_lists(list1, total_list)
        total_list = self.merge_lists(list2, total_list)

        i = 0
        while i < len(total_list):
            for elem in total_list:
                if elem.find(total_list[i]) > - 1 and elem != total_list[i]:
                    total_list.remove(total_list[i])
                    continue
            i += 1

        return total_list

    def return_bird_name(self, sentence):
        if len(sentence)>0:
            sentence = UserSentence(sentence)
            bd_ = BirdDetection()
            # Check if the bird can be found in any list.
            return_bird_list = bd_.find_bird_from_list(sentence.preprocessed_text)

            # Run the model.
            bird_list_ner = bd_.run_model(sentence.preprocessed_text)

            return_bird_list = self.consolidate_bird_list(return_bird_list, bird_list_ner)

            return return_bird_list
        else:
            return "No text found."

    def __init__(self):
        pass


from IndiAvesBirdIdentification.exim import Exim

import re
import demoji
import preprocessor as p
import spacy
nlp = spacy.load("en_core_web_sm")
p.set_options(p.OPT.EMOJI, p.OPT.MENTION, p.OPT.URL, p.OPT.SMILEY, p.OPT.NUMBER, p.OPT.HASHTAG)


class UserSentence:
    text = ""
    preprocessed_text = ""
    spelling_corrections = {"grey": "gray", "pegion": "pigeon", "brested": "breasted", "serpant": "serpent",
                            "avedavat": "avadavat", "open billed stork": "asian openbill",
                            "secretary bird": "Secretarybird", "dollar bird": "dollarbird", "silver bill": "silverbill",
                            "eyes": "eye"}

    def get_bird_name_from_hashtag_4levels(self, hashtag_, all_birds_list):
        hashtag_ = hashtag_.lower()
        special_cases = {"greateradjutantstork": "greateradjutant"}

        for key in special_cases:
            if hashtag_ == key: hashtag_ = special_cases[key]

        rel_birdnames = []
        for bird in all_birds_list:
            if bird[-2:] == hashtag_[-2:] and hashtag_[:2] == bird[:2]:
                rel_birdnames.append(bird)

        if len(rel_birdnames) > 0:
            if hashtag_ in rel_birdnames:
                return hashtag_
            segments = [0, 1, 2, 3]
            m_ = 2
            while m_ < len(hashtag_) - 2:
                segments[0] = hashtag_[:m_]
                n_ = 0
                while n_ < len(hashtag_[m_:]):
                    segments[1] = hashtag_[m_:][:n_]
                    part3 = hashtag_[m_:][n_:]
                    o_ = 0
                    while o_ < len(hashtag_[m_:][n_:]):
                        segments[2] = hashtag_[m_:][n_:][:o_]
                        p_ = 0
                        while p_ < len(hashtag_[m_:][n_:][o_:]):
                            segments[3] = hashtag_[m_:][n_:][o_:][:p_]
                            part4 = hashtag_[m_:][n_:][o_:][p_:]
                            prob_birdname = segments[0] + " " + segments[1] + " " + segments[2] + " " + segments[3] + " " + part4
                            prob_birdname = re.sub(r' +', ' ', prob_birdname)
                            if prob_birdname in rel_birdnames:
                                return prob_birdname
                            p_ += 1
                        o_ += 1
                    n_ += 1
                m_ += 1
        return None

    def remove_emojis(self, sentence):
        emojis = demoji.findall(sentence)
        for item in emojis:
            sentence = sentence.replace(item, " " + emojis[item] + " ")
        return sentence

    def replace_underscores(self, sentence):
        sentence = sentence.lower()
        sentence = sentence.replace("_", " ")
        return sentence

    def try_replacing_hashtags_mit_birdname(self, text, all_birds_list):
        status = False
        hashtags = re.findall(r"#(\w+)", text)
        for hashtag in hashtags:
            segmented_ = self.get_bird_name_from_hashtag_4levels(hashtag, all_birds_list)
            if segmented_ is not None: text = text.replace("#" + hashtag, segmented_)
        return text

    def basic_preprocess(self, sentence):
        sentence = p.clean(sentence)
        if sentence[:2] == "b'":
            sentence = sentence[1:]
        sentence = re.sub(r'[^\w\s]', ' ', sentence)
        sentence = re.sub(r' +', ' ', sentence)
        sentence = sentence.strip()
        for key in self.spelling_corrections:
            if sentence.find(key) > -1:
                sentence = sentence.replace(key, self.spelling_corrections[key])
        return sentence

    def plural_nn_to_singular(self, sentence, birdnames_words):
        doc = nlp(sentence)
        for token in doc:
            if token.pos_ == "NOUN":
                if token.text[-1:] == "s" and token.text not in birdnames_words:
                    sentence = sentence.replace(token.text, token.text[:-1])
        return sentence

    def preprocess(self, text):
        exim = Exim()
        text = self.remove_emojis(text)  # Removes Emojis
        text = self.replace_underscores(text)  # Replaces underscores
        text = self.try_replacing_hashtags_mit_birdname(text, exim.all_birds_list)
        text = self.basic_preprocess(text)
        text = self.plural_nn_to_singular(text, exim.birdnames_words)
        return text

    def __init__(self, text_):
        self.text = text_
        self.preprocessed_text = self.preprocess(text_)


from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '0.1'
DESCRIPTION = 'Detecting bird name from short text (Tweets)'
LONG_DESCRIPTION = 'Detecting bird name from user generated short text over social media. The model is trained with tweets pulled with #IndiAves'

# Setting up
setup(
    name="IndiAvesBirdIdentification",
    version=VERSION,
    author="Anirban Saha",
    author_email="<mailme@anirbansaha.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(), 
    package_data={}, #new
    include_package_data=True, #new
    install_requires=['tweet-preprocessor', 'spacy', 'demoji', 'tweepy'],
    keywords=['python', 'birding', 'bird name', 'twitter', 'bird name detection'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)
